@@ -9,10 +9,6 @@ import pyclipper
 from .ext.cavaliercontours import cavaliercontours as cavc
 from .vc_types import VcObject
 
-# import trimesh
-# import pocketing
-# from shapely import geometry
-
 TWO_PI = math.pi * 2
 
 
@@ -488,46 +484,6 @@ def found_next_tab_point(mpos, offsets):
     return ()
 
 
-"""
-def do_pockets_trochoidal(  # pylint: disable=R0913
-    polyline,
-    obj,
-    obj_idx,
-    tool_offset,
-    tool_radius,
-    polyline_offsets,
-    offset_idx,
-    vertex_data_org,
-):
-    plist = []
-    vertex_data = polyline.vertex_data()
-    for pos_x, pos_y in zip(vertex_data[0], vertex_data[1]):
-        plist.append([pos_x, pos_y])
-
-    poly = geometry.Polygon(plist)
-
-    #toolpaths = pocketing.contour.contour_parallel(poly, tool_radius)
-    toolpaths = [pocketing.trochoidal.toolpath(poly, step=(tool_radius * 0.75))]
-
-    for part in toolpaths:
-        points = []
-        for point in part:
-            points.append(point.tolist())
-        vertex_data = points2vertex(points)
-        polyline_offset = cavc.Polyline(vertex_data, is_closed=obj.closed)
-        polyline_offset.level = len(obj.outer_objects)
-        polyline_offset.tool_offset = tool_offset
-        polyline_offset.layer = obj.layer
-        polyline_offset.setup = obj.setup
-        polyline_offset.start = obj.start
-        polyline_offset.is_pocket = True
-        polyline_offsets[f"{obj_idx}.{offset_idx}"] = polyline_offset
-        offset_idx += 1
-
-    return offset_idx
-"""
-
-
 def points2offsets(
     obj,
     obj_idx,
@@ -536,6 +492,7 @@ def points2offsets(
     offset_idx,
     tool_offset,
     scale=1.0,
+    is_pocket=0,
 ):
     vertex_data = points2vertex(points, scale=scale)
     polyline_offset = cavc.Polyline(vertex_data, is_closed=obj.closed)
@@ -544,7 +501,8 @@ def points2offsets(
     polyline_offset.layer = obj.layer
     polyline_offset.setup = obj.setup
     polyline_offset.start = obj.start
-    polyline_offset.is_pocket = True
+    polyline_offset.is_pocket = is_pocket
+    polyline_offset.fixed_direction = False
     polyline_offsets[f"{obj_idx}.{offset_idx}"] = polyline_offset
     offset_idx += 1
     return offset_idx
@@ -561,6 +519,7 @@ def do_pockets(  # pylint: disable=R0913
     vertex_data_org,  # pylint: disable=W0613
 ):
     """calculates multiple offset lines of an polyline"""
+    abs_tool_radius = abs(tool_radius)
     if obj.inner_objects and obj.setup["pockets"]["islands"]:
         subjs = []
         vertex_data = vertex_data_cache(polyline)
@@ -582,7 +541,7 @@ def do_pockets(  # pylint: disable=R0913
                     pyclipper.JT_ROUND,  # pylint: disable=E1101
                     pyclipper.ET_CLOSEDPOLYGON,  # pylint: disable=E1101
                 )
-        subjs = pco.Execute(-tool_radius * 100)
+        subjs = pco.Execute(-abs_tool_radius * 100)
 
         for points in subjs:
             offset_idx = points2offsets(
@@ -593,6 +552,7 @@ def do_pockets(  # pylint: disable=R0913
                 offset_idx,
                 tool_offset,
                 scale=0.01,
+                is_pocket=2,
             )
 
         while True:
@@ -603,10 +563,9 @@ def do_pockets(  # pylint: disable=R0913
                     pyclipper.JT_ROUND,  # pylint: disable=E1101
                     pyclipper.ET_CLOSEDPOLYGON,  # pylint: disable=E1101
                 )
-            subjs = pco.Execute(-tool_radius * 100)  # pylint: disable=E1101
+            subjs = pco.Execute(-abs_tool_radius * 100)  # pylint: disable=E1101
             if not subjs:
                 break
-
             for points in subjs:
                 offset_idx = points2offsets(
                     obj,
@@ -616,6 +575,7 @@ def do_pockets(  # pylint: disable=R0913
                     offset_idx,
                     tool_offset,
                     scale=0.01,
+                    is_pocket=2,
                 )
 
     elif obj.segments[0]["type"] == "CIRCLE" and "center" in obj.segments[0]:
@@ -625,14 +585,14 @@ def do_pockets(  # pylint: disable=R0913
         points = []
         rad = 0
         while True:
-            rad += tool_radius / 2
-            if rad > radius - tool_radius:
+            rad += abs_tool_radius / 2
+            if rad > radius - abs_tool_radius:
                 break
-            points.append((center[0] - rad, center[1] + 0.1, 1.0))
-            rad += tool_radius / 2
-            if rad > radius - tool_radius:
+            points.append((center[0] - rad, center[1] + 0.01, 1.0))
+            rad += abs_tool_radius / 2
+            if rad > radius - abs_tool_radius:
                 break
-            points.append((center[0] + rad, center[1] - 0.1, 1.0))
+            points.append((center[0] + rad, center[1] - 0.01, 1.0))
         vertex_data = points2vertex(points)
         polyline_offset = cavc.Polyline(vertex_data, is_closed=False)
         polyline_offset.level = len(obj.outer_objects)
@@ -640,14 +600,13 @@ def do_pockets(  # pylint: disable=R0913
         polyline_offset.layer = obj.layer
         polyline_offset.setup = obj.setup
         polyline_offset.start = obj.start
-        polyline_offset.is_pocket = True
+        polyline_offset.is_pocket = 1
+        polyline_offset.fixed_direction = True
         polyline_offsets[f"{obj_idx}.{offset_idx}"] = polyline_offset
         offset_idx += 1
 
     else:
-        offsets = polyline.parallel_offset(
-            delta=-tool_radius, check_self_intersect=True
-        )
+        offsets = polyline.parallel_offset(delta=tool_radius, check_self_intersect=True)
         for polyline_offset in offsets:
             if polyline_offset:
                 # workaround for bad offsets
@@ -660,7 +619,8 @@ def do_pockets(  # pylint: disable=R0913
                 polyline_offset.layer = obj.layer
                 polyline_offset.setup = obj.setup
                 polyline_offset.start = obj.start
-                polyline_offset.is_pocket = True
+                polyline_offset.is_pocket = 1
+                polyline_offset.fixed_direction = False
                 polyline_offsets[f"{obj_idx}.{offset_idx}"] = polyline_offset
                 offset_idx += 1
                 if polyline_offset.is_closed():
@@ -772,7 +732,8 @@ def object2polyline_offsets(
             over_polyline.start = obj.start
             over_polyline.setup = obj.setup
             over_polyline.layer = obj.layer
-            over_polyline.is_pocket = False
+            over_polyline.is_pocket = 0
+            over_polyline.fixed_direction = False
             over_polyline.tool_offset = tool_offset
             polyline_offsets[f"{obj_idx}.{offset_idx}"] = over_polyline
 
@@ -794,7 +755,7 @@ def object2polyline_offsets(
     offset_idx = 0
     if polyline.is_closed() and tool_offset != "none":
         polyline_offset_list = polyline.parallel_offset(
-            delta=-tool_radius, check_self_intersect=True
+            delta=tool_radius, check_self_intersect=True
         )
         if polyline_offset_list:
             for polyline_offset in polyline_offset_list:
@@ -805,7 +766,8 @@ def object2polyline_offsets(
                 polyline_offset.tool_offset = tool_offset
                 polyline_offset.setup = obj.setup
                 polyline_offset.layer = obj.layer
-                polyline_offset.is_pocket = False
+                polyline_offset.is_pocket = 0
+                polyline_offset.fixed_direction = False
                 polyline_offset.is_circle = is_circle
                 polyline_offsets[f"{obj_idx}.{offset_idx}"] = polyline_offset
                 offset_idx += 1
@@ -833,7 +795,8 @@ def object2polyline_offsets(
             polyline_offset.tool_offset = tool_offset
             polyline_offset.setup = obj.setup
             polyline_offset.layer = obj.layer
-            polyline_offset.is_pocket = False
+            polyline_offset.is_pocket = 0
+            polyline_offset.fixed_direction = False
             polyline_offset.is_circle = True
             polyline_offsets[f"{obj_idx}.{offset_idx}.x"] = polyline_offset
             offset_idx += 1
@@ -847,7 +810,8 @@ def object2polyline_offsets(
         polyline.tool_offset = tool_offset
         polyline.start = obj.start
         polyline.layer = obj.layer
-        polyline.is_pocket = False
+        polyline.is_pocket = 0
+        polyline.fixed_direction = False
         polyline.is_circle = False
         polyline_offsets[f"{obj_idx}.{offset_idx}"] = polyline
         offset_idx += 1
@@ -868,7 +832,7 @@ def objects2polyline_offsets(diameter, objects, max_outer, small_circles=False):
 
             obj_copy = deepcopy(obj)
             do_reverse = 0
-            if obj_copy.tool_offset == "inside":
+            if obj_copy.tool_offset == "outside":
                 do_reverse = 1 - do_reverse
 
             if obj_copy["setup"]["mill"]["reverse"]:
