@@ -11,6 +11,8 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
         self.rate: int = 0
         self.speed: int = -1
         self.offsets: tuple[float, float, float] = (0.0, 0.0, 0.0)
+        self.offsets_reset = False
+        self.scale: float = 1.0
 
     def separation(self) -> None:
         if self.comments:
@@ -28,11 +30,13 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
                 self.gcode.append("G21 (Metric/mm)")
             else:
                 self.gcode.append("G21")
+            self.scale = 1.0
         else:
             if self.comments:
                 self.gcode.append("G20 (Imperial/inches)")
             else:
                 self.gcode.append("G20")
+            self.scale = 1.0 / 25.4
 
     def absolute(self, active=True) -> None:
         if active:
@@ -63,17 +67,24 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
                 self.gcode.append("G42")
 
     def machine_offsets(
-        self, offsets: tuple[float, float, float] = (0.0, 0.0, 0.0)
+        self, offsets: tuple[float, float, float] = (0.0, 0.0, 0.0), soft: bool = True
     ) -> None:
-        self.offsets = offsets
-        if self.offsets != (0.0, 0.0, 0.0):
+        self.offsets_reset = False
+        if not soft and offsets != (0.0, 0.0, 0.0):
+            self.offsets_reset = True
             self.gcode.append(
                 f"G10 L2 P1 X{offsets[0]} Y{offsets[1]} Z{offsets[2]} (workpiece offsets for G54)"
             )
             self.gcode.append("G54")
+        if soft:
+            self.offsets = (
+                offsets[0] / self.scale,
+                offsets[1] / self.scale,
+                offsets[2] / self.scale,
+            )
 
     def program_end(self) -> None:
-        if self.offsets != (0.0, 0.0, 0.0):
+        if self.offsets_reset:
             self.gcode.append("G10 L2 P1 X0 Y0 Z0 (reset offsets for G54)")
         self.gcode.append("M02")
 
@@ -84,13 +95,13 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
     def move(self, x_pos=None, y_pos=None, z_pos=None) -> None:
         line = []
         if x_pos is not None and self.x_pos != x_pos:
-            line.append(f"X{round(x_pos, 6)}")
+            line.append(f"X{round((x_pos + self.offsets[0]) * self.scale, 6)}")
             self.x_pos = x_pos
         if y_pos is not None and self.y_pos != y_pos:
-            line.append(f"Y{round(y_pos, 6)}")
+            line.append(f"Y{round((y_pos + self.offsets[1]) * self.scale, 6)}")
             self.y_pos = y_pos
         if z_pos is not None and self.z_pos != z_pos:
-            line.append(f"Z{round(z_pos, 6)}")
+            line.append(f"Z{round((z_pos + self.offsets[2]) * self.scale, 6)}")
             self.z_pos = z_pos
         if line:
             self.gcode.append("G00 " + " ".join(line))
@@ -137,13 +148,13 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
     def linear(self, x_pos=None, y_pos=None, z_pos=None) -> None:
         line = []
         if x_pos is not None and self.x_pos != x_pos:
-            line.append(f"X{round(x_pos, 6)}")
+            line.append(f"X{round((x_pos + self.offsets[0]) * self.scale, 6)}")
             self.x_pos = x_pos
         if y_pos is not None and self.y_pos != y_pos:
-            line.append(f"Y{round(y_pos, 6)}")
+            line.append(f"Y{round((y_pos + self.offsets[1]) * self.scale, 6)}")
             self.y_pos = y_pos
         if z_pos is not None and self.z_pos != z_pos:
-            line.append(f"Z{round(z_pos, 6)}")
+            line.append(f"Z{round((z_pos + self.offsets[2]) * self.scale, 6)}")
             self.z_pos = z_pos
         if line:
             self.gcode.append("G01 " + " ".join(line))
@@ -153,13 +164,13 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
     ) -> None:
         line = []
         if x_pos is not None and self.x_pos != x_pos:
-            line.append(f"X{round(x_pos, 6)}")
+            line.append(f"X{round((x_pos + self.offsets[0]) * self.scale, 6)}")
             self.x_pos = x_pos
         if y_pos is not None and self.y_pos != y_pos:
-            line.append(f"Y{round(y_pos, 6)}")
+            line.append(f"Y{round((y_pos + self.offsets[1]) * self.scale, 6)}")
             self.y_pos = y_pos
         if z_pos is not None and self.z_pos != z_pos:
-            line.append(f"Z{round(z_pos, 6)}")
+            line.append(f"Z{round((z_pos + self.offsets[2]) * self.scale, 6)}")
             self.z_pos = z_pos
         if i_pos is not None:
             line.append(f"I{round(i_pos, 5)}")
@@ -175,13 +186,13 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
     ) -> None:
         line = []
         if x_pos is not None and self.x_pos != x_pos:
-            line.append(f"X{round(x_pos, 6)}")
+            line.append(f"X{round((x_pos + self.offsets[0]) * self.scale, 6)}")
             self.x_pos = x_pos
         if y_pos is not None and self.y_pos != y_pos:
-            line.append(f"Y{round(y_pos, 6)}")
+            line.append(f"Y{round((y_pos + self.offsets[1]) * self.scale, 6)}")
             self.y_pos = y_pos
         if z_pos is not None and self.z_pos != z_pos:
-            line.append(f"Z{round(z_pos, 6)}")
+            line.append(f"Z{round((z_pos + self.offsets[2]) * self.scale, 6)}")
             self.z_pos = z_pos
         if i_pos is not None:
             line.append(f"I{round(i_pos, 5)}")
