@@ -171,6 +171,13 @@ def setup_defaults(_) -> dict:
                     "lenght": "float",
                     "blades": "int",
                 },
+                "column_defaults": {
+                    "name": "",
+                    "number": 99,
+                    "diameter": 1.0,
+                    "lenght": 1.0,
+                    "blades": 1,
+                },
                 "title": _("Tools"),
                 "tooltip": _("tooltable"),
             },
@@ -237,6 +244,13 @@ def setup_defaults(_) -> dict:
                     "fz8": "float",
                     "fz12": "float",
                 },
+                "column_defaults": {
+                    "name": "",
+                    "vc": 100,
+                    "fz4": 0.05,
+                    "fz8": 0.05,
+                    "fz12": 0.05,
+                },
                 "title": _("Materials"),
                 "tooltip": _("materialtable"),
             },
@@ -379,6 +393,22 @@ def setup_defaults(_) -> dict:
                 "title": _("Tool-Mode"),
                 "tooltip": _("Tool-Mode"),
             },
+            "unit": {
+                "default": "mm",
+                "type": "select",
+                "options": (
+                    ("mm", _("mm")),
+                    ("inch", _("inch")),
+                ),
+                "title": _("Unit"),
+                "tooltip": _("Unit of the maschine"),
+            },
+            "g54": {
+                "default": False,
+                "type": "bool",
+                "title": _("maschine supports g54"),
+                "tooltip": _("maschine supports g54"),
+            },
             "comments": {
                 "default": True,
                 "type": "bool",
