@@ -95,21 +95,25 @@ for reader in ("dxfread", "hpglread", "stlread", "svgread", "ttfread", "imgread"
         )
         reader_plugins[reader] = drawing_reader.DrawReader
     except Exception as reader_error:  # pylint: disable=W0703
-        print(f"ERRO while loading input plugin {reader}: {reader_error}")
+        sys.stderr.write(f"ERRO while loading input plugin {reader}: {reader_error}\n")
 
 
 DEBUG = False
 TIMESTAMP = 0
 
 
+def eprint(message, *args, **kwargs):  # pylint: disable=W0613
+    sys.stderr.write(f"{message}\n")
+
+
 def debug(message):
     global TIMESTAMP  # pylint: disable=W0603
     if DEBUG:
         now = time.time()
         if TIMESTAMP == 0:
             TIMESTAMP = now
-        print(round(now - TIMESTAMP, 1))
-        print(f"{message} ", end="", flush=True)
+        eprint(round(now - TIMESTAMP, 1))
+        eprint(f"{message} ", end="", flush=True)
         TIMESTAMP = now
 
 
@@ -126,10 +130,11 @@ if lang:
         lang_translations = gettext.translation(
             "base", localedir=localedir, languages=[lang]
         )
+
         lang_translations.install()
         _ = lang_translations.gettext
     except FileNotFoundError:
-        print(f"WARNING: localedir not found {localedir}")
+        sys.stderr.write(f"WARNING: localedir not found {localedir}\n")
 
 
 class GLWidget(QGLWidget):
@@ -282,41 +287,49 @@ class GLWidget(QGLWidget):
 
         GL.glPopMatrix()
 
-    def toggle_tab_selector(self) -> None:
+    def toggle_tab_selector(self) -> bool:
         self.selection = ()
         if self.selector_mode == "":
             self.selector_mode = "tab"
             self.view_2d()
-        else:
+            return True
+        if self.selector_mode == "tab":
             self.selector_mode = ""
             self.view_reset()
+        return False
 
-    def toggle_start_selector(self) -> None:
+    def toggle_start_selector(self) -> bool:
         self.selection = ()
         if self.selector_mode == "":
             self.selector_mode = "start"
             self.view_2d()
-        else:
+            return True
+        if self.selector_mode == "start":
             self.selector_mode = ""
             self.view_reset()
+        return False
 
-    def toggle_repair_selector(self) -> None:
+    def toggle_repair_selector(self) -> bool:
         self.selection = ()
         if self.selector_mode == "":
             self.selector_mode = "repair"
             self.view_2d()
-        else:
+            return True
+        if self.selector_mode == "repair":
             self.selector_mode = ""
             self.view_reset()
+        return False
 
-    def toggle_delete_selector(self) -> None:
+    def toggle_delete_selector(self) -> bool:
         self.selection = ()
         if self.selector_mode == "":
             self.selector_mode = "delete"
             self.view_2d()
-        else:
+            return True
+        if self.selector_mode == "delete":
             self.selector_mode = ""
             self.view_reset()
+        return False
 
     def view_2d(self) -> None:
         """toggle view function."""
@@ -590,10 +603,11 @@ class ViaConstructor:
                         )
             for vport in doc.viewports.get_config("*Active"):  # type: ignore
                 vport.dxf.grid_on = True
-            ezdxf.zoom.extents(msp)  # type: ignore
+            if hasattr(ezdxf, "zoom"):
+                ezdxf.zoom.extents(msp)  # type: ignore
             doc.saveas(output_file)
         except Exception as error:  # pylint: disable=W0703
-            print(f"ERROR while saving dxf: {error}")
+            eprint(f"ERROR while saving dxf: {error}")
             return False
 
         return True
@@ -629,8 +643,12 @@ class ViaConstructor:
         debug("run_calculation: offsets")
 
         # create toolpath from objects
+        unit = psetup["maschine"]["unit"]
+        diameter = psetup["tool"]["diameter"]
+        if unit == "inch":
+            diameter *= 25.4
         self.project["offsets"] = objects2polyline_offsets(
-            psetup["tool"]["diameter"],
+            diameter,
             self.project["objects"],
             self.project["maxOuter"],
             psetup["mill"]["small_circles"],
@@ -652,7 +670,7 @@ class ViaConstructor:
             self.project["suffix"] = output_plugin.suffix()
             self.project["axis"] = output_plugin.axis()
         else:
-            print(
+            eprint(
                 f"ERROR: Unknown maschine output plugin: {self.project['setup']['maschine']['plugin']}"
             )
             sys.exit(1)
@@ -703,19 +721,47 @@ class ViaConstructor:
 
     def _toolbar_toggle_tab_selector(self) -> None:
         """tab selector."""
-        self.project["glwidget"].toggle_tab_selector()
+        title = "Tab-Selector"
+        if self.project["glwidget"].toggle_tab_selector():
+            for toolbutton in self.toolbuttons.values():
+                if toolbutton[5]:
+                    toolbutton[7].setChecked(False)
+            self.toolbuttons[_(title)][7].setChecked(True)
+        else:
+            self.toolbuttons[_(title)][7].setChecked(False)
 
     def _toolbar_toggle_delete_selector(self) -> None:
         """delete selector."""
-        self.project["glwidget"].toggle_delete_selector()
+        title = "Delete-Selector"
+        if self.project["glwidget"].toggle_delete_selector():
+            for toolbutton in self.toolbuttons.values():
+                if toolbutton[5]:
+                    toolbutton[7].setChecked(False)
+            self.toolbuttons[_(title)][7].setChecked(True)
+        else:
+            self.toolbuttons[_(title)][7].setChecked(False)
 
     def _toolbar_toggle_start_selector(self) -> None:
         """start selector."""
-        self.project["glwidget"].toggle_start_selector()
+        title = "Start-Selector"
+        if self.project["glwidget"].toggle_start_selector():
+            for toolbutton in self.toolbuttons.values():
+                if toolbutton[5]:
+                    toolbutton[7].setChecked(False)
+            self.toolbuttons[_(title)][7].setChecked(True)
+        else:
+            self.toolbuttons[_(title)][7].setChecked(False)
 
     def _toolbar_toggle_repair_selector(self) -> None:
         """start selector."""
-        self.project["glwidget"].toggle_repair_selector()
+        title = "Repair-Selector"
+        if self.project["glwidget"].toggle_repair_selector():
+            for toolbutton in self.toolbuttons.values():
+                if toolbutton[5]:
+                    toolbutton[7].setChecked(False)
+            self.toolbuttons[_(title)][7].setChecked(True)
+        else:
+            self.toolbuttons[_(title)][7].setChecked(False)
 
     def _toolbar_view_reset(self) -> None:
         """center view."""
@@ -727,7 +773,7 @@ class ViaConstructor:
             fd_machine_cmd.write("\n")
             if self.project["setup"]["maschine"]["postcommand"]:
                 cmd = f"{self.project['setup']['maschine']['postcommand']} '{filename}'"
-                print(f"executing postcommand: {cmd}")
+                eprint(f"executing postcommand: {cmd}")
                 os.system(f"{cmd} &")
             return True
         return False
@@ -736,7 +782,7 @@ class ViaConstructor:
         if self.status_bar:
             self.status_bar.showMessage(message)
         else:
-            print(f"STATUS: {message}")
+            eprint(f"STATUS: {message}")
 
     def _toolbar_save_machine_cmd(self) -> None:
         """save machine_cmd."""
@@ -898,9 +944,7 @@ class ViaConstructor:
         GL.glEndList()
         self.info = f"{self.project['minMax'][2] - self.project['minMax'][0]}x{self.project['minMax'][3] - self.project['minMax'][1]}mm"
         if self.main:
-            self.main.setWindowTitle(
-                f"viaConstructor: {self.project['filename_draw']} - {self.info}"
-            )
+            self.main.setWindowTitle("viaConstructor")
         self.status_bar_message(f"{self.info} - calculate..done")
 
     def materials_select(self, material_idx) -> None:
@@ -911,6 +955,9 @@ class ViaConstructor:
         maschine_toolspeed = self.project["setup"]["maschine"]["tool_speed"]
         tool_number = self.project["setup"]["tool"]["number"]
         tool_diameter = self.project["setup"]["tool"]["diameter"]
+        unit = self.project["setup"]["maschine"]["unit"]
+        if unit == "inch":
+            tool_diameter *= 25.4
         tool_vc = self.project["setup"]["tool"]["materialtable"][material_idx]["vc"]
         tool_speed = tool_vc * 1000 / (tool_diameter * math.pi)
         tool_speed = int(min(tool_speed, maschine_toolspeed))
@@ -996,7 +1043,7 @@ class ViaConstructor:
         elif entry_type == "table":
             pass
         else:
-            print(f"Unknown setup-type: {entry_type}")
+            eprint(f"Unknown setup-type: {entry_type}")
             value = None
         self.project["objects"][obj_idx]["setup"][sname][ename] = value
         self.project["maxOuter"] = find_tool_offsets(self.project["objects"])
@@ -1097,7 +1144,7 @@ class ViaConstructor:
                         elif entry["type"] == "table":
                             pass
                         else:
-                            print(f"Unknown setup-type: {entry['type']}")
+                            eprint(f"Unknown setup-type: {entry['type']}")
         # self.project["objwidget"].expandAll()
         debug("update_table: done")
 
@@ -1188,7 +1235,7 @@ class ViaConstructor:
                                 ] = float(value)
                             col_idx += 1
                 else:
-                    print(f"Unknown setup-type: {entry['type']}")
+                    eprint(f"Unknown setup-type: {entry['type']}")
 
         if self.project["setup"]["mill"]["step"] >= 0.0:
             self.project["setup"]["mill"]["step"] = -0.05
@@ -1240,176 +1287,36 @@ class ViaConstructor:
 
     def _toolbar_exit(self) -> None:
         """exit button."""
+        if os.environ.get("LINUXCNCVERSION"):
+            eprint(self.project["machine_cmd"])
         sys.exit(0)
 
     def create_toolbar(self) -> None:
         """creates the_toolbar."""
-        toolbuttons = {
-            _("Load drawing"): (
-                "open.png",
-                "",
-                _("Load drawing"),
-                self._toolbar_load_drawing,
-                True,
-                "main",
-            ),
-            _("Save drawing as DXF"): (
-                "save.png",
-                "Ctrl+S",
-                _("Save drawing as DXF"),
-                self._toolbar_save_dxf,
-                True,
-                "main",
-            ),
-            _("Exit"): (
-                "exit.png",
-                "Ctrl+Q",
-                _("Exit application"),
-                self._toolbar_exit,
-                True,
-                "main",
-            ),
-            _("Save Machine-Commands"): (
-                "save-gcode.png",
-                "Ctrl+S",
-                _("Save machine commands"),
-                self._toolbar_save_machine_cmd,
-                True,
-                "machine_cmd",
-            ),
-            _("Load setup from"): (
-                "load-setup.png",
-                "",
-                _("Load setup from"),
-                self._toolbar_load_setup_from,
-                True,
-                "setup",
-            ),
-            _("Load setup from machine_cmd"): (
-                "load-setup-gcode.png",
-                "",
-                _("Load-Setup from machine_cmd"),
-                self._toolbar_load_machine_cmd_setup,
-                False,  # os.path.isfile(self.project["filename_machine_cmd"]),
-                "setup",
-            ),
-            _("Save setup as default"): (
-                "save-setup.png",
-                "",
-                _("Save-Setup"),
-                self._toolbar_save_setup,
-                True,
-                "setup",
-            ),
-            _("Save setup as"): (
-                "save-setup-as.png",
-                "",
-                _("Save setup  as"),
-                self._toolbar_save_setup_as,
-                True,
-                "setup",
-            ),
-            _("View-Reset"): (
-                "view-reset.png",
-                "",
-                _("View-Reset"),
-                self._toolbar_view_reset,
-                True,
-                "view",
-            ),
-            _("2D-View"): (
-                "view-2d.png",
-                "",
-                _("2D-View"),
-                self._toolbar_view_2d,
-                True,
-                "view",
-            ),
-            _("Flip-X"): (
-                "flip-x.png",
-                "",
-                _("Flip-X workpiece"),
-                self._toolbar_flipx,
-                True,
-                "workpiece",
-            ),
-            _("Flip-Y"): (
-                "flip-y.png",
-                "",
-                _("Flip-Y workpiece"),
-                self._toolbar_flipy,
-                True,
-                "workpiece",
-            ),
-            _("Rotate"): (
-                "rotate.png",
-                "",
-                _("Rotate workpiece"),
-                self._toolbar_rotate,
-                True,
-                "workpiece",
-            ),
-            _("Scale"): (
-                "scale.png",
-                "",
-                _("Scale workpiece"),
-                self._toolbar_scale,
-                True,
-                "workpiece",
-            ),
-            _("Tab-Selector"): (
-                "tab-selector.png",
-                "",
-                _("Tab-Selector"),
-                self._toolbar_toggle_tab_selector,
-                True,
-                "tabs",
-            ),
-            _("Start-Selector"): (
-                "start.png",
-                "",
-                _("Start-Selector"),
-                self._toolbar_toggle_start_selector,
-                True,
-                "start",
-            ),
-            _("Repair-Selector"): (
-                "repair.png",
-                "",
-                _("Repair-Selector"),
-                self._toolbar_toggle_repair_selector,
-                True,
-                "repair",
-            ),
-            _("Delete-Selector"): (
-                "delete.png",
-                "",
-                _("Delete-Selector"),
-                self._toolbar_toggle_delete_selector,
-                True,
-                "delete",
-            ),
-        }
         self.toolbar = QToolBar("top toolbar")
         self.main.addToolBar(self.toolbar)  # type: ignore
         section = ""
 
-        for title, toolbutton in toolbuttons.items():
+        for title, toolbutton in self.toolbuttons.items():
             icon = os.path.join(self.module_root, "icons", toolbutton[0])
-            if toolbutton[5] != section:
+            if toolbutton[6] != section:
                 self.toolbar.addSeparator()
-                section = toolbutton[5]
+                section = toolbutton[6]
             if toolbutton[4]:
                 action = QAction(
                     QIcon(icon),
                     title,
                     self.main,
                 )
+                if toolbutton[5]:
+                    action.setCheckable(True)
+                action.triggered.connect(toolbutton[3])  # type: ignore
+
                 if toolbutton[1]:
                     action.setShortcut(toolbutton[1])
                 action.setStatusTip(toolbutton[2])
-                action.triggered.connect(toolbutton[3])  # type: ignore
                 self.toolbar.addAction(action)
+                toolbutton[7] = action
 
     def update_global_setup(self) -> None:
         for sname in self.project["setup_defaults"]:
@@ -1425,9 +1332,53 @@ class ViaConstructor:
                 elif entry["type"] == "str":
                     entry["widget"].setText(self.project["setup"][sname][ename])
                 elif entry["type"] == "table":
-                    pass
+                    # add empty row if not exist
+                    first_element = list(entry["columns"].keys())[0]
+                    if (
+                        str(self.project["setup"][sname][ename][-1][first_element])
+                        != ""
+                    ):
+                        new_row = {}
+                        for key, default in entry["column_defaults"].items():
+                            new_row[key] = default
+                        self.project["setup"][sname][ename].append(new_row)
+
+                    table = entry["widget"]
+                    table.setRowCount(len(self.project["setup"][sname][ename]))
+                    idxf_offset = 0
+                    table.setColumnCount(len(entry["columns"]))
+                    if entry["selectable"]:
+                        table.setColumnCount(len(entry["columns"]) + 1)
+                        table.setHorizontalHeaderItem(0, QTableWidgetItem("Select"))
+                        idxf_offset = 1
+                    for col_idx, title in enumerate(entry["columns"]):
+                        table.setHorizontalHeaderItem(
+                            col_idx + idxf_offset, QTableWidgetItem(title)
+                        )
+                    for row_idx, row in enumerate(self.project["setup"][sname][ename]):
+                        if entry["selectable"]:
+                            button = QPushButton()
+                            button.setIcon(
+                                QIcon(
+                                    os.path.join(
+                                        self.module_root, "icons", "select.png"
+                                    )
+                                )
+                            )
+                            button.setToolTip(_("select this row"))
+                            button.clicked.connect(partial(self.table_select, sname, ename, row_idx))  # type: ignore
+                            table.setCellWidget(row_idx, 0, button)
+                            table.resizeColumnToContents(0)
+                        for col_idx, key in enumerate(entry["columns"]):
+                            table.setItem(
+                                row_idx,
+                                col_idx + idxf_offset,
+                                QTableWidgetItem(str(row[key])),
+                            )
+                            table.resizeColumnToContents(col_idx + idxf_offset)
+
                 else:
-                    print(f"Unknown setup-type: {entry['type']}")
+                    eprint(f"Unknown setup-type: {entry['type']}")
 
     def create_global_setup(self, tabwidget) -> None:
         for sname in self.project["setup_defaults"]:
@@ -1482,6 +1433,17 @@ class ViaConstructor:
                     hlayout.addWidget(lineedit)
                     entry["widget"] = lineedit
                 elif entry["type"] == "table":
+                    # add empty row if not exist
+                    first_element = list(entry["columns"].keys())[0]
+                    if (
+                        str(self.project["setup"][sname][ename][-1][first_element])
+                        != ""
+                    ):
+                        new_row = {}
+                        for key, default in entry["column_defaults"].items():
+                            new_row[key] = default
+                        self.project["setup"][sname][ename].append(new_row)
+
                     table = QTableWidget()
                     label.setToolTip(entry.get("tooltip", f"{sname}/{ename}"))
                     table.setRowCount(len(self.project["setup"][sname][ename]))
@@ -1520,7 +1482,7 @@ class ViaConstructor:
                     vlayout.addWidget(table)
                     entry["widget"] = table
                 else:
-                    print(f"Unknown setup-type: {entry['type']}")
+                    eprint(f"Unknown setup-type: {entry['type']}")
 
     def udate_tabs_data(self) -> None:
         self.project["tabs"]["data"] = []
@@ -1625,7 +1587,7 @@ class ViaConstructor:
             self.project["setup"]["view"]["path"] = "minimal"
             self.project["setup"]["view"]["object_ids"] = False
 
-        print(f"ERROR: can not load file: {filename}")
+        eprint(f"ERROR: can not load file: {filename}")
         debug("load_drawing: error")
         return False
 
@@ -1666,6 +1628,207 @@ class ViaConstructor:
 
         self.args = parser.parse_args()
 
+        if os.environ.get("LINUXCNCVERSION"):
+            self.toolbuttons = {
+                _("Exit"): [
+                    "exit.png",
+                    "Ctrl+Q",
+                    _("Exit application"),
+                    self._toolbar_exit,
+                    True,
+                    False,
+                    "main",
+                    None,
+                ],
+            }
+        else:
+            self.toolbuttons = {
+                _("Load drawing"): [
+                    "open.png",
+                    "",
+                    _("Load drawing"),
+                    self._toolbar_load_drawing,
+                    True,
+                    False,
+                    "main",
+                    None,
+                ],
+                _("Save drawing as DXF"): [
+                    "save.png",
+                    "Ctrl+S",
+                    _("Save drawing as DXF"),
+                    self._toolbar_save_dxf,
+                    True,
+                    False,
+                    "main",
+                    None,
+                ],
+                _("Exit"): [
+                    "exit.png",
+                    "Ctrl+Q",
+                    _("Exit application"),
+                    self._toolbar_exit,
+                    True,
+                    False,
+                    "main",
+                    None,
+                ],
+                _("Save Machine-Commands"): [
+                    "save-gcode.png",
+                    "Ctrl+S",
+                    _("Save machine commands"),
+                    self._toolbar_save_machine_cmd,
+                    True,
+                    False,
+                    "machine_cmd",
+                    None,
+                ],
+            }
+        self.toolbuttons.update(
+            {
+                _("Load setup from"): [
+                    "load-setup.png",
+                    "",
+                    _("Load setup from"),
+                    self._toolbar_load_setup_from,
+                    True,
+                    False,
+                    "setup",
+                    None,
+                ],
+                _("Load setup from machine_cmd"): [
+                    "load-setup-gcode.png",
+                    "",
+                    _("Load-Setup from machine_cmd"),
+                    self._toolbar_load_machine_cmd_setup,
+                    False,  # os.path.isfile(self.project["filename_machine_cmd"]),
+                    False,
+                    "setup",
+                    None,
+                ],
+                _("Save setup as default"): [
+                    "save-setup.png",
+                    "",
+                    _("Save-Setup"),
+                    self._toolbar_save_setup,
+                    True,
+                    False,
+                    "setup",
+                    None,
+                ],
+                _("Save setup as"): [
+                    "save-setup-as.png",
+                    "",
+                    _("Save setup  as"),
+                    self._toolbar_save_setup_as,
+                    True,
+                    False,
+                    "setup",
+                    None,
+                ],
+                _("View-Reset"): [
+                    "view-reset.png",
+                    "",
+                    _("View-Reset"),
+                    self._toolbar_view_reset,
+                    True,
+                    False,
+                    "view",
+                    None,
+                ],
+                _("2D-View"): [
+                    "view-2d.png",
+                    "",
+                    _("2D-View"),
+                    self._toolbar_view_2d,
+                    True,
+                    False,
+                    "view",
+                    None,
+                ],
+                _("Flip-X"): [
+                    "flip-x.png",
+                    "",
+                    _("Flip-X workpiece"),
+                    self._toolbar_flipx,
+                    True,
+                    False,
+                    "workpiece",
+                    None,
+                ],
+                _("Flip-Y"): [
+                    "flip-y.png",
+                    "",
+                    _("Flip-Y workpiece"),
+                    self._toolbar_flipy,
+                    True,
+                    False,
+                    "workpiece",
+                    None,
+                ],
+                _("Rotate"): [
+                    "rotate.png",
+                    "",
+                    _("Rotate workpiece"),
+                    self._toolbar_rotate,
+                    True,
+                    False,
+                    "workpiece",
+                    None,
+                ],
+                _("Scale"): [
+                    "scale.png",
+                    "",
+                    _("Scale workpiece"),
+                    self._toolbar_scale,
+                    True,
+                    False,
+                    "workpiece",
+                    None,
+                ],
+                _("Tab-Selector"): [
+                    "tab-selector.png",
+                    "",
+                    _("Tab-Selector"),
+                    self._toolbar_toggle_tab_selector,
+                    True,
+                    True,
+                    "tabs",
+                    None,
+                ],
+                _("Start-Selector"): [
+                    "start.png",
+                    "",
+                    _("Start-Selector"),
+                    self._toolbar_toggle_start_selector,
+                    True,
+                    True,
+                    "start",
+                    None,
+                ],
+                _("Repair-Selector"): [
+                    "repair.png",
+                    "",
+                    _("Repair-Selector"),
+                    self._toolbar_toggle_repair_selector,
+                    True,
+                    True,
+                    "repair",
+                    None,
+                ],
+                _("Delete-Selector"): [
+                    "delete.png",
+                    "",
+                    _("Delete-Selector"),
+                    self._toolbar_toggle_delete_selector,
+                    True,
+                    True,
+                    "delete",
+                    None,
+                ],
+            }
+        )
+
         # load setup
         debug("main: load setup")
         self.project["setup"] = {}
@@ -1683,12 +1846,12 @@ class ViaConstructor:
             # save and exit
             if self.args.dxf:
                 self.update_drawing()
-                print("saving dawing to file:", self.args.dxf)
+                eprint(f"saving dawing to file: {self.args.dxf}")
                 self.save_objects_as_dxf(self.args.dxf)
                 sys.exit(0)
             if self.args.output:
                 self.update_drawing()
-                print("saving machine_cmd to file:", self.args.output)
+                eprint(f"saving machine_cmd to file: {self.args.output}")
                 open(self.args.output, "w").write(self.project["machine_cmd"])
                 sys.exit(0)
 
@@ -1712,7 +1875,7 @@ class ViaConstructor:
         self.project["glwidget"] = GLWidget(self.project, self.update_drawing)
 
         self.main = QMainWindow()
-        self.main.setWindowTitle(f"viaConstructor: {self.project['filename_draw']}")
+        self.main.setWindowTitle("viaConstructor")
         self.main.setCentralWidget(self.project["window"])
 
         self.this_dir, self.this_filename = os.path.split(__file__)
